@@ -414,20 +414,34 @@ Examples:
         (handler-case (let ((*in-a-finalizer* t)) (funcall fun))
           (error (c) (warn "Error calling finalizer ~S:~%  ~A" fun c)))))))
 
-#+sb-thread (define-alien-variable finalizer-thread-runflag int)
+#+sb-thread
+(progn
+(define-alien-variable finalizer-thread-runflag int)
+;; post-GC hooks are synchronously invoked after GC if #-sb-thread
+(define-load-time-global *after-gc-hooks* nil)
+(define-load-time-global *run-gc-hooks* 0)
+(declaim (fixnum *run-gc-hooks*)))
+
 ;;; Drain the queue of finalizers and return when empty.
 ;;; Concurrent invocations of this function in different threads are ok.
 ;;; Nested invocations (from a GC forced by a finalizer) are not ok.
 ;;; See the trace at the bottom of this file.
-(define-load-time-global *bg-compiler-function* nil)
+(define-load-time-global *bg-compiler-function* #'sb-c::default-compiler-worker)
 (defun run-pending-finalizers (&aux (system-finalizer-scratchpad (list 0)))
   (declare (dynamic-extent system-finalizer-scratchpad))
   (finalizers-rehash)
   (loop
-   ;; Perform no further work if trying to stop the thread, even if there is work.
-   #+sb-thread (when (zerop finalizer-thread-runflag) (return))
+   #+sb-thread
+   (progn
+     ;; Perform no further work if trying to stop the thread, even if there is work.
+     (when (zerop finalizer-thread-runflag) (return))
+     ;; If this thread is very slow to notice *RUN-GC-HOOKS* due to a slow finalizer,
+     ;; requests to run can stack up. A request will not be lost but may be delayed
+     (when (plusp *run-gc-hooks*)
+       (sb-vm:without-arena (call-hooks "after-GC" *after-gc-hooks* :on-error :warn))
+       (atomic-decf *run-gc-hooks*)))
    (let ((ran-bg-compile ; Try to run a background compilation task
-          (when *bg-compiler-function* (funcall *bg-compiler-function*)))
+          (funcall *bg-compiler-function*))
          (ran-a-system-finalizer ; Try to run 1 system finalizer
           (sb-vm::immobile-code-dealloc-1 system-finalizer-scratchpad))
          (ran-a-user-finalizer ; Try to run 1 user finalizer
@@ -440,8 +454,10 @@ Examples:
 (declaim (type (or sb-thread:thread (eql :start) null) *finalizer-thread*))
 #+sb-thread
 (progn
-(defun finalizer-thread-notify ()
-  (alien-funcall (extern-alien "finalizer_thread_wake" (function void)))
+(defun finalizer-thread-notify (run-hooks)
+  (declare (bit run-hooks))
+  (alien-funcall (extern-alien "finalizer_thread_wake" (function void int))
+                 run-hooks)
   nil)
 
 ;;; The following operations are synchronized by *MAKE-THREAD-LOCK* -
@@ -461,6 +477,9 @@ Examples:
             (lambda ()
               (setf *finalizer-thread* sb-thread:*current-thread*)
               (loop (run-pending-finalizers)
+                    ;; a slight race here- technically the finalizer thread should not
+                    ;; put itself to sleep unless it has ascertained that there is no
+                    ;; background work while it holds a mutex on the various queues.
                     (alien-funcall (extern-alien "finalizer_thread_wait" (function void)))
                     (when (zerop finalizer-thread-runflag) (return)))
               (setq *finalizer-thread* nil))
