@@ -1585,8 +1585,8 @@ variable: an unreadable object representing the error is printed instead.")
                  (scale (r s m+ m-)
                    (let ((est (truly-the (integer -323 309)
                                          (ceiling (- (* (+ e (integer-length (truly-the sb-kernel:double-float-significand f)) -1)
-                                                        (log $2d0 10))
-                                                     $1.0e-10)))))
+                                                        (log 2d0 10))
+                                                     1.0e-10)))))
                      (if (>= est 0)
                          (fixup r (* s (expt10 est)) m+ m- est)
                          (let ((scale (expt10 (- est))))
@@ -1750,42 +1750,34 @@ variable: an unreadable object representing the error is printed instead.")
 ;;; denormalized, we must pull out a large factor, since there is more
 ;;; negative exponent range than positive range.
 
-(eval-when (:compile-toplevel :execute)
-  (setf *read-default-float-format*
-        #+long-float 'cl:long-float #-long-float 'cl:double-float))
 (defun scale-exponent (original-x)
   (let* ((x (coerce original-x 'long-float)))
     (multiple-value-bind (sig exponent) (decode-float x)
       (declare (ignore sig))
-      (if (= x $0.0e0)
-          (values (float $0.0e0 original-x) 1)
+      (if (= x 0.0l0)
+          (values (float 0.0l0 original-x) 1)
           (let* ((ex (locally (declare (optimize (safety 0)))
                        (the fixnum
-                         (round (* exponent
-                                   #-long-float (log $2d0 10)
-                                   #+long-float
-                                   (error "(log 2 10) not computed"))))))
+                         (round (* exponent (log 2l0 10))))))
                  (x (if (minusp ex)
                         (if (float-denormalized-p x)
                             #-long-float
-                            (* x $1.0e16 (expt $10.0e0 (- (- ex) 16)))
+                            (* x 1.0l16 (expt 10.0l0 (- (- ex) 16)))
                             #+long-float
-                            (* x $1.0e18 (expt $10.0e0 (- (- ex) 18)))
-                            (* x $10.0e0 (expt $10.0e0 (- (- ex) 1))))
-                        (/ x $10.0e0 (expt $10.0e0 (1- ex))))))
-            (do ((d $10.0e0 (* d $10.0e0))
+                            (* x 1.0l18 (expt 10.0l0 (- (- ex) 18)))
+                            (* x 10.0l0 (expt 10.0l0 (- (- ex) 1))))
+                        (/ x 10.0l0 (expt 10.0l0 (1- ex))))))
+            (do ((d 10.0l0 (* d 10.0l0))
                  (y x (/ x d))
                  (ex ex (1+ ex)))
-                ((< y $1.0e0)
-                 (do ((m $10.0e0 (* m $10.0e0))
+                ((< y 1.0l0)
+                 (do ((m 10.0l0 (* m 10.0l0))
                       (z y (* y m))
                       (ex ex (1- ex)))
-                     ((>= z $0.1e0)
+                     ((>= z 0.1l0)
                       (values (float z original-x) ex))
                    (declare (long-float m) (integer ex))))
               (declare (long-float d))))))))
-(eval-when (:compile-toplevel :execute)
-  (setf *read-default-float-format* 'cl:single-float))
 
 ;;;; entry point for the float printer
 
@@ -1917,7 +1909,7 @@ variable: an unreadable object representing the error is printed instead.")
              (write-string "broken weak pointer" stream))))))
 
 (defmethod print-object ((component code-component) stream)
-  (print-unreadable-object (component stream :identity t)
+  (print-unreadable-object (component stream)
     (let (dinfo)
       (cond ((eq (setq dinfo (%code-debug-info component)) :bpt-lra)
              (write-string "bpt-trap-return" stream))
@@ -1933,7 +1925,10 @@ variable: an unreadable object representing the error is printed instead.")
                (cond ((not (typep dinfo 'sb-c::debug-info)))
                      ((neq (sb-c::debug-info-name dinfo) fun-name)
                       (write-string ", " stream)
-                      (output-object (sb-c::debug-info-name dinfo) stream)))))))))
+                      (output-object (sb-c::debug-info-name dinfo) stream)))))))
+    (let ((a (get-lisp-obj-address component)))
+      (format stream " {~X..~X}"
+              a (+ (logandc2 a sb-vm:lowtag-mask) (code-object-size component))))))
 
 #-(or x86 x86-64 arm64 riscv)
 (defmethod print-object ((lra lra) stream)
