@@ -247,19 +247,6 @@
 
 ;;; Access to the GENCGC page table for better precision in
 ;;; MAP-ALLOCATED-OBJECTS
-#+immobile-space
-(progn
-    (define-alien-type nil
-        ;; ... and yet another place for Lisp to become out-of-sync with C.
-        (struct immobile-page
-                (flags (unsigned 8))
-                (obj-spacing (unsigned 8))
-                (obj-size (unsigned 8))
-                (generations (unsigned 8))
-                (free-index (unsigned 32))
-                (page-link (unsigned 16))
-                (prior-free-index (unsigned 16))))
-    (define-alien-variable "fixedobj_pages" (* (struct immobile-page))))
 (define-alien-variable "next_free_page" sb-kernel::page-index-t)
 
 #+immobile-space
@@ -953,31 +940,9 @@ We could try a few things to mitigate this:
                   ;; between layout bitmap indices and indices as given to
                   ;; FUNCALLABLE-INSTANCE-INFO is not so obvious, and it's
                   ;; both tricky and unnecessary to generalize iteration.
-                  ;; So just hardcode the few cases that exist.
-                  #+compact-instance-header
-                  (ecase (layout-bitmap (%fun-layout ,obj))
-                    (-1 ; external trampoline, all slots are tagged
-                     ;; In this case, the trampoline word is scanned, with no ill effect.
-                     (loop for .i. from 0
-                           to (- (get-closure-length ,obj) funcallable-instance-info-offset)
-                           do (,functoid (%funcallable-instance-info ,obj .i.) ,@more)))
-                    (#b0110 ; internal trampoline, 2 raw slots, 1 tagged slot
-                     ;;   ^ ---- trampoline
-                     ;;  ^------ implementation function
-                     ;; ^------- (FUNCALLABLE-INSTANCE-INFO 0)
-                     ;; and the rest of the words are raw.
-                     (,functoid (%funcallable-instance-info ,obj 0) ,@more)))
-                  #-compact-instance-header
-                  (progn
-                    (aver (eql (layout-bitmap (%fun-layout ,obj)) -4))
-                    ;;            v ----trampoline
-                    ;; = #b1...1100
-                    ;;           ^----- layout
-                    ;;          ^------ implementation function
-                    ;;         ^------- (FUNCALLABLE-INSTANCE-INFO 0)
-                    (loop for .i. from 0
-                          to (- (get-closure-length ,obj) funcallable-instance-info-offset)
-                          do (,functoid (%funcallable-instance-info ,obj .i.) ,@more)))))
+                  (loop for .i. from 0
+                        to (- (get-closure-length ,obj) funcallable-instance-info-offset)
+                        do (,functoid (%funcallable-instance-info ,obj .i.) ,@more))))
             .,(make-case 'function))) ; in case there was code provided for it
          (t
           ;; TODO: the generated code is pretty horrible. OTHER-POINTER-LOWTAG
@@ -1001,22 +966,7 @@ We could try a few things to mitigate this:
                `(,functoid (symbol-package ,obj) ,@more))
             ,.(make-case 'fdefn
                `(fdefn-name ,obj)
-               `(fdefn-fun ,obj)
-               ;; While it looks like we could easily allow a pointer to a movable object
-               ;; in the fdefn-raw-addr slot, it is not exactly trivial- at a bare minimum,
-               ;; translating the raw-addr to a lispobj might have to be pseudoatomic,
-               ;; since we don't know what object to pin when reconstructing it.
-               ;; For simple-funs in dynamic space, it doesn't have to be pseudoatomic
-               ;; because a reference to the interior of code pins the code.
-               ;; Closure trampolines would be fine as well. That leaves funcallable instances
-               ;; as the pain point. Those could go on pages of code as well, but see the
-               ;; comment in conservative_root_p() in gencgc as to why that alone
-               ;; would be inadequate- we require a properly tagged descriptor
-               ;; to enliven any object other than code.
-               #+(and immobile-code x86-64)
-               `(%make-lisp-obj
-                 (alien-funcall (extern-alien "decode_fdefn_rawfun" (function unsigned unsigned))
-                                (logandc2 (get-lisp-obj-address ,obj) lowtag-mask))))
+               `(fdefn-fun ,obj))
             ,.(make-case* 'code-component
                `(loop for .i. from 2 below (code-header-words ,obj)
                       do (,functoid (code-header-ref ,obj .i.) ,@more)))
@@ -1355,15 +1305,17 @@ We could try a few things to mitigate this:
     (map-objects-in-range #'print-it (%make-lisp-obj start) (%make-lisp-obj end)))))
 
 (defun map-code-objects (fun)
+  (declare (dynamic-extent fun))
   (dx-flet ((filter (obj type size)
               (declare (ignore size))
               (when (= type code-header-widetag)
                 (funcall fun obj))))
     (without-gcing
       #+immobile-code
-      (map-objects-in-range #'filter
-                            (ash text-space-start (- n-fixnum-tag-bits))
-                            (%make-lisp-obj (sap-int *text-space-free-pointer*)))
+      (with-system-mutex (*allocator-mutex*)
+        (map-objects-in-range #'filter
+                              (ash text-space-start (- n-fixnum-tag-bits))
+                              (%make-lisp-obj (sap-int *text-space-free-pointer*))))
       (alien-funcall (extern-alien "close_code_region" (function void)))
       (walk-dynamic-space #'filter
                           #b1111111 ; all generations
