@@ -82,7 +82,6 @@
   (def set-header-data (x val))
   (def widetag-of)
   (def %other-pointer-widetag)
-  (def pointer-hash)
   (def vector-sap)
   (def binding-stack-pointer-sap  ())
   #+cheneygc (def dynamic-space-free-pointer ())
@@ -91,13 +90,14 @@
   (def %fun-pointer-widetag)
   (def %closure-fun)
   (def %closure-index-ref (closure index))
+  (def make-fdefn)
   (def fdefn-name)
   (def fdefn-fun)
   (def fdefn-makunbound)
   (def sb-c::vector-length)
   (def make-array-header (type rank))
   (def code-instructions)
-  #-untagged-fdefns (def code-header-ref (code-obj index))
+  (def code-header-ref (code-obj index))
   (def %vector-raw-bits (object offset))
   (def %set-vector-raw-bits (object offset value))
   #-weak-vector-readbarrier (def weak-vector-len)
@@ -157,7 +157,7 @@
         (%make-simd-pack-256-double (a b c d))
         (%make-simd-pack-256-ub64 (a b c d))
         (%simd-pack-256-tag))
-  #+sb-thread (def sb-vm::current-thread-offset-sap)
+  #+(or sb-thread x86-64) (def sb-vm::current-thread-offset-sap)
   (def current-sp ())
   (def current-fp ())
   (def stack-ref (s n))
