@@ -195,7 +195,7 @@
 (defun add-equality-constraint (operator first second constraints consequent-constraints alternative-constraints
                                 &optional (min-amount 0) max-amount)
   (case operator
-    ((vector-length vector-length<=)
+    ((vector-length vector-length<= vector-length>=)
      (let ((var (if (lambda-var-p first)
                     first
                     (lvar-dest-var first))))
@@ -203,6 +203,7 @@
                   (lambda-var-constraints var))
          (add-equality-constraint (case operator
                                     (vector-length<= '<=)
+                                    (vector-length>= '>=)
                                     (t 'eq))
                                   (make-vector-length-constraint var) second
                                   constraints consequent-constraints alternative-constraints))))
@@ -462,12 +463,12 @@
     (map-equality-constraints x y gen
                               (lambda (op not-p)
                                 (case op
-                                  ((eq eql char=)
+                                  ((eq char=)
                                    (return (not not-p)))
                                   ((> <)
                                    (unless not-p
                                      (return nil)))
-                                  ((= <= >= char-equal)
+                                  ((= <= >= char-equal eql)
                                    (when not-p
                                      (return nil))))))
     :give-up))
@@ -772,9 +773,9 @@
         do
         (add-equality-constraint operator var second constraints target nil min-amount max-amount))
   (when (lambda-var-p var)
-    (let ((vector-length (vector-length-var-p lvar constraints)))
+    (let ((vector-length (vector-length-var-p lvar constraints t)))
       (when vector-length
-        (conset-add-equality-constraint target 'eq var vector-length nil)))))
+        (add-equality-constraint 'eq var vector-length constraints target nil)))))
 
 (defun add-mv-let-result-constraints (call fun constraints &optional (target constraints))
   (let ((vars (lambda-vars fun))
@@ -906,7 +907,28 @@
              c)))
     c))
 
-(defoptimizer (sb-kernel:vector-subseq* constraint-propagate-result) ((sequence start end) node gen)
+(defun subseq-bounds (sequence start end gen)
+  (let* ((null-p (types-equal-or-intersect (lvar-type end) (specifier-type 'null)))
+         (end (if null-p
+                  (type-union (type-intersection (or (vector-length-type (lvar-type sequence))
+                                                     *universal-type*)
+                                                 (or (vector-constraint-length sequence gen)
+                                                     *universal-type*))
+                              (type-intersection (lvar-type end) (specifier-type 'integer)))
+                  (lvar-type end)))
+         (int-s (type-approximate-interval (lvar-type start)))
+         (int-e (type-approximate-interval end)))
+    (if (and int-s int-e)
+        (let* ((length (interval-sub int-e int-s))
+               (l (interval-low length))
+               (h (interval-high length)))
+          (values (if (typep l 'unsigned-byte)
+                      l
+                      0)
+                  h))
+        (values 0 nil))))
+
+(defoptimizer (vector-subseq* constraint-propagate-result) ((sequence start end) node gen)
   (let (c
         (null-p (types-equal-or-intersect (lvar-type end) (specifier-type 'null))))
     (when (eql (lvar-type start) (specifier-type '(eql 0)))
@@ -916,28 +938,70 @@
               (when seq-var
                 (push (list 'vector-length (make-vector-length-constraint seq-var)) c))))
           (push (list 'vector-length end) c)))
-    (let* ((end (if null-p
-                    (type-union (type-intersection (or (vector-length-type (lvar-type sequence))
-                                                       *universal-type*)
-                                                   (or (vector-constraint-length sequence gen)
-                                                       *universal-type*))
-                                (type-intersection (lvar-type end) (specifier-type 'integer)))
-                    (lvar-type end)))
-           (int-s (type-approximate-interval (lvar-type start)))
-           (int-e (type-approximate-interval end)))
-      (when (and int-s int-e)
-        (let* ((length (interval-sub int-e int-s))
-               (l (interval-low length))
-               (h (interval-high length)))
-          (push (list 'vector-length (specifier-type
-                                      `(integer ,(if (typep l 'unsigned-byte)
-                                                     l
-                                                     0)
-                                                ,(or h
-                                                     '*))))
-                c))))
+    (multiple-value-bind (l h) (subseq-bounds sequence start end gen)
+      (when (or h
+                (> l 0))
+        (push (list 'vector-length (specifier-type `(integer ,l ,(or h '*))))
+              c)))
     c))
 
+(defun concatenate-constraints (args gen &optional subseq)
+  (let ((min-sum 0)
+        (max t)
+        (max-sum 0)
+        r)
+    (loop while args
+          do (let ((arg (pop args)))
+               (cond ((and subseq
+                           (constant-lvar-p arg)
+                           (eq (lvar-value arg) 'sb-impl::%subseq))
+                      (multiple-value-bind (l h) (subseq-bounds (pop args) (pop args)
+                                                                (pop args) gen)
+                        (incf min-sum l)
+                        (if h
+                            (incf max-sum h)
+                            (setf max nil))))
+                     (t
+                      (let ((int (type-approximate-interval (type-intersection (or (vector-length-type (lvar-type arg))
+                                                                                   *universal-type*)
+                                                                               (or (vector-constraint-length arg gen)
+                                                                                   *universal-type*)))))
+                        (cond (int
+                               (incf min-sum (or (interval-low int) 0))
+                               (if (interval-high int)
+                                   (incf max-sum (interval-high int))
+                                   (setf max nil)))
+                              (t
+                               (setf max nil))))))))
+    (when (plusp min-sum)
+      (push (list 'vector-length>= (specifier-type `(eql ,min-sum)))
+            r))
+    (when (and max
+               (plusp max-sum))
+      (push (list 'vector-length<= (specifier-type `(eql ,max-sum)))
+            r))
+    r))
+
+(defoptimizers constraint-propagate-result
+    (%concatenate-to-simple-vector %concatenate-to-string %concatenate-to-base-string)
+    ((&rest seqs) node gen)
+  (concatenate-constraints seqs gen))
+
+(defoptimizers constraint-propagate-result
+    (%concatenate-to-vector)
+    ((widetag &rest seqs) node gen)
+  (concatenate-constraints seqs gen))
+
+(defoptimizers constraint-propagate-result
+    (%concatenate-to-string-subseq %concatenate-to-base-string-subseq %concatenate-to-simple-vector-subseq)
+    ((&rest seqs) node gen)
+  (concatenate-constraints seqs gen t))
+
+(defoptimizers constraint-propagate-result
+    (%concatenate-to-vector-subseq)
+    ((type &rest seqs) node gen)
+  (concatenate-constraints seqs gen t))
+
 (defoptimizer (read-sequence constraint-propagate-result) ((seq stream &key start end) node gen)
   (let ((var (ok-lvar-lambda-var seq gen)))
     (when var
@@ -1089,7 +1153,7 @@
                                    (equal-length-constraints x y gen)))
           (values nil nil constraints))))))
 
-(defoptimizer (equalp constraint-propagate-if) ((x y) node gen)
+(defoptimizers constraint-propagate-if (equalp bit-vector-= sb-impl::array-equalp) ((x y) node gen)
   (let ((x-type (equalp-type (lvar-type x)))
         (y-type (equalp-type (lvar-type y))))
     (let ((intersection (type-intersection x-type y-type)))
@@ -1203,3 +1267,9 @@
     (unless (or start1 start2 end1 end2)
       (setf constraints (equality-length-constraints '<= sub-sequence1 main-sequence2 gen)))
     (values nil nil constraints)))
+
+(defoptimizer (%other-pointer-p constraint-propagate-if) ((x))
+  (values x (specifier-type 'other-pointer)))
+
+(defoptimizer (%array-rank= constraint-propagate-if) ((x n))
+  (values x (specifier-type `(array * ,(lvar-value n)))))
