@@ -87,13 +87,19 @@
   ;; as a consistency checking mechanism inside the compiler during IR2
   ;; transformation.
   always-translatable
-  ;; Function's funarg can safely skip its argument count check.
-  callee-omit-arg-count-check
   ;; If a function is called with two arguments and the first one is a
   ;; constant, then the arguments will be swapped.
   commutative
   ;; Reoptimize this function if the node that follows it gets unlinked.
-  reoptimize-when-unlinking)
+  reoptimize-when-unlinking
+  ;; The function does not verify the arg count and must be always
+  ;; called with the right arguments and can avoid passing NARGS.
+  no-verify-arg-count
+  ;; Arguments are can be passed unboxed, no type checking on entry is
+  ;; performed, and the number of arguments passed in registers can be
+  ;; greater than the standard number. Only fixed arguments can be used.
+  fixed-args
+  unboxed-return)
 
 (defstruct (fun-info (:copier nil)
                      #-sb-xc-host (:pure t))
@@ -144,6 +150,10 @@
   ;; each constraint flips the meaning of the constraint if it is
   ;; non-NIL.
   (constraint-propagate nil :type (or function null))
+  ;; Propagating stuff back to the arguments based on the constraints on
+  ;; the result of this combination.
+  (constraint-propagate-back nil :type (or function null))
+  (constraint-propagate-result nil :type (or function null))
   ;; If true, the function can add flow-sensitive type information
   ;; depending on the truthiness of its return value.  Returns two
   ;; values, a LVAR and a CTYPE. The LVAR is of that CTYPE iff the
@@ -164,7 +174,11 @@
   (result-arg nil :type (or index null))
   ;; Customizing behavior of ASSERT-CALL-TYPE
   (call-type-deriver nil :type (or function null))
-  annotation)
+  annotation
+  ;; For functions with unboxed args/returns
+  (folder nil :type (or function null))
+  (externally-checkable-type nil :type (or function null))
+  (constants nil :type (or function null)))
 
 (defprinter (fun-info)
   (attributes :test (not (zerop attributes))
