@@ -20,6 +20,43 @@
 (defconstant old-fp-passing-offset
   (make-sc+offset control-stack-sc-number ocfp-save-offset))
 
+(defun linkage-cell-fixup (name node)
+  ;; The distinction between the two linkage-cell fixups is that :linkage-cell
+  ;; never warns about undefined linkage but the "-ud" one may, after resolving
+  ;; separately-compiled fasls much the same as a standard linker, thereby liberating
+  ;; SBCL from the restriction that WITH-COMPILATION-UNIT is the only way to avoid
+  ;; style-warnings about defined-elsewhere functions. Needless to say, we must avoid
+  ;; emitting any such warnings at compile-time. Two tests are done to decide whether
+  ;; to emit the load-time warning:
+  ;; - did the compiler style-warn?  If not then don't. This covers the case
+  ;;   of (FUNCALL 'name)
+  ;; - was it lexically notinline? Even if the compiler style-warned, there should
+  ;;   not be a load-time warning if the user wanted an out-of-line call.
+  (let* ((explicit-notinline
+          (sb-c::fun-lexically-notinline-p name (sb-c::node-lexenv node)))
+         (lt-warn ; (possible) load-time warning for unresolved linkage
+          (and (not explicit-notinline)
+               ;; Optimize the predicate to FIND-IF. We can usually
+               ;; compare names by EQ except when NAME is a list.
+               (find-if (if (symbolp name)
+                            (lambda (x)
+                              (and (eq (sb-c::undefined-warning-kind x) :function)
+                                   (eq (sb-c::undefined-warning-name x) name)))
+                            (lambda (x)
+                              (and (eq (sb-c::undefined-warning-kind x) :function)
+                                   (equal (sb-c::undefined-warning-name x) name))))
+                        sb-c::*undefined-warnings*))))
+    (make-fixup name (if lt-warn
+                         :linkage-cell-ud  ; was undefined at compile-time
+                         :linkage-cell))))
+
+(defun compute-linkage-cell (node name res)
+  (cond ((sb-c::code-immobile-p node)
+         (inst lea res (rip-relative-ea (linkage-cell-fixup name node))))
+        (t
+         (inst mov res (thread-slot-ea sb-vm::thread-linkage-table-slot))
+         (inst lea res (ea (linkage-cell-fixup name node) res)))))
+
 ;;; Make the TNs used to hold OLD-FP and RETURN-PC within the current
 ;;; function. We treat these specially so that the debugger can find
 ;;; them at a known location.
@@ -259,9 +296,9 @@
                       (not verify))))
      (flet ((check-nargs ()
               (assemble ()
-                (let* ((*location-context* (list* name
-                                                  (type-specifier type)
-                                                  (make-restart-location SKIP)))
+                (let* ((*location-context* (list* (make-restart-location SKIP)
+                                                  name
+                                                  (type-specifier type)))
                        (err-lab (generate-error-code vop 'invalid-arg-count-error))
                        (min min-values)
                        (max (and (< max-values call-arguments-limit)
@@ -657,16 +694,11 @@
 ;;; have been set up in the current frame.
 (defmacro define-full-call (vop-name named return variable &optional args)
   (aver (not (and variable (eq return :tail))))
-  #+immobile-code (when named (setq named :direct))
   `(define-vop (,vop-name ,@(when (eq return :unknown) '(unknown-values-receiver)))
      (:args    ,@(unless (eq return :tail)
                    '((new-fp :scs (any-reg) :to (:argument 1))))
 
-               ;; If immobile-space is in use, then named call does not require
-               ;; a register unless the caller is NOT in immobile space,
-               ;; in which case the register is needed because there is no
-               ;; absolute addressing mode for jmp/call.
-               ,@(unless (eq named :direct)
+               ,@(unless named ; FUN is an info argument for named call
                    '((fun :scs (descriptor-reg control-stack)
                           :target rax :to (:argument 0))))
 
@@ -692,29 +724,28 @@
                ;; Intuitively you might want FUN to be the first codegen arg,
                ;; but that won't work, because EMIT-ARG-MOVES wants the
                ;; passing locs in (FIRST (vop-codegen-info vop)).
-               ,@(when (eq named :direct) '(fun))
+               ,@(when named '(fun))
                ,@(when (eq return :fixed) '(nvals))
                step-instrumenting
                ,@(unless named '(fun-type)))
 
      (:ignore   ,@(unless (or variable (eq return :tail)) '(arg-locs))
                 ,@(unless variable '(args))
-                ,@(when (eq return :unboxed) '(values)))
+                ,@(when (eq return :unboxed) '(values))
+                ,@(when (eq args :fixed) '(nargs)))
 
-               ;; We pass either the fdefn object (for named call) or
-               ;; the actual function object (for unnamed call) in
-               ;; RAX. With named call, closure-tramp will replace it
-               ;; with the real function and invoke the real function
-               ;; for closures. Non-closures do not need this value,
-               ;; so don't care what shows up in it.
-     ,@(unless (eq named :direct)
-         '((:temporary (:sc descriptor-reg :offset rax-offset
-                        :from (:argument 0) :to :eval) rax)))
+     ;; For anonymous call, RAX is the function. For named call, RAX will be the linkage
+     ;; table base if not stepping, or the linkage cell itself if stepping.
+     ;; Calls from immobile-space without stepping avoid using RAX, and instead
+     ;; access the linkage table relative to RIP.
+     (:temporary (:sc descriptor-reg :offset rax-offset :from (:argument 0) :to :eval) rax)
 
      ;; We pass the number of arguments in RCX.
-     (:temporary
-      (:sc unsigned-reg :offset rcx-offset :to ,(if (eq return :fixed) :save :eval))
-      rcx)
+     ,@(unless (eq args :fixed)
+         `((:temporary
+            (:sc unsigned-reg :offset rcx-offset
+             :to ,(if (eq return :fixed) :save :eval))
+            rcx)))
 
      ,@(when (eq return :fixed)
                    ;; Save it for DEFAULT-UNKNOWN-VALUES to work
@@ -750,26 +781,26 @@
                ;; This has to be done before the frame pointer is
                ;; changed! RAX stores the 'lexical environment' needed
                ;; for closures.
-       ,@(unless (eq named :direct) '((move rax fun)))
-
-       ,@(if variable
-                     ;; For variable call, compute the number of
-                     ;; arguments and move some of the arguments to
-                     ;; registers.
-             `((inst mov rcx new-fp)
-               (inst sub rcx rsp-tn)
-               (inst shr rcx ,(- word-shift n-fixnum-tag-bits))
-                              ;; Move the necessary args to registers,
-                              ;; this moves them all even if they are
-                              ;; not all needed.
-               ,@(loop for name in *register-arg-names*
-                       for index downfrom -1
-                       collect `(loadw ,name new-fp ,index)))
-             '((cond ((listp nargs)) ;; no-verify-arg-count
-                     ((zerop nargs)
-                      (zeroize rcx))
-                     (t
-                      (inst mov rcx (fixnumize nargs))))))
+       ,@(unless named '((move rax fun)))
+       ,@(unless (eq args :fixed)
+           (if variable
+               ;; For variable call, compute the number of
+               ;; arguments and move some of the arguments to
+               ;; registers.
+               `((inst mov rcx new-fp)
+                 (inst sub rcx rsp-tn)
+                 (inst shr rcx ,(- word-shift n-fixnum-tag-bits))
+                 ;; Move the necessary args to registers,
+                 ;; this moves them all even if they are
+                 ;; not all needed.
+                 ,@(loop for name in *register-arg-names*
+                         for index downfrom -1
+                         collect `(loadw ,name new-fp ,index)))
+               '((cond ((listp nargs)) ;; no-verify-arg-count
+                       ((zerop nargs)
+                        (zeroize rcx))
+                       (t
+                        (inst mov rcx (fixnumize nargs)))))))
        ,@(cond ((eq return :tail)
                 '(        ;; Python has figured out what frame we should
                           ;; return to so might as well use that clue.
@@ -827,25 +858,16 @@
                    (storew rbp-tn new-fp (frame-word-offset ocfp-save-offset))
                    (move rbp-tn new-fp))))  ; NB - now on new stack frame.
 
-       (when (and step-instrumenting
-                  ,@(and (eq named :direct)
-                         `((not (and #+immobile-code
-                                     ;; handle-single-step-around-trap can't handle it
-                                     (static-fdefn-offset fun))))))
+       (when step-instrumenting
+         ,@(when named '((compute-linkage-cell node fun rax)))
          (emit-single-step-test)
          (inst jmp :eq DONE)
          (inst break single-step-around-trap))
        DONE
        (note-this-location vop :call-site)
-       ,(cond ((eq named :direct)
-                       #+immobile-code `(emit-direct-call fun ',(if (eq return :tail) 'jmp 'call)
-                                                          node step-instrumenting)
-                       #-immobile-code `(inst ,(if (eq return :tail) 'jmp 'call)
-                                              (ea (+ nil-value (static-fun-offset fun)))))
-              #-immobile-code
-              (named
-               `(inst ,(if (eq return :tail) 'jmp 'call)
-                      (object-slot-ea rax fdefn-raw-addr-slot other-pointer-lowtag)))
+       ,(cond (named
+               `(emit-direct-call fun ',(if (eq return :tail) 'jmp 'call)
+                                  node step-instrumenting))
               ((eq return :tail)
                `(tail-call-unnamed rax fun-type vop))
               (t
@@ -859,16 +881,10 @@
 
 (define-full-call call nil :fixed nil)
 (define-full-call call-named t :fixed nil)
-#-immobile-code
-(define-full-call static-call-named :direct :fixed nil)
 (define-full-call multiple-call nil :unknown nil)
 (define-full-call multiple-call-named t :unknown nil)
-#-immobile-code
-(define-full-call static-multiple-call-named :direct :unknown nil)
 (define-full-call tail-call nil :tail nil)
 (define-full-call tail-call-named t :tail nil)
-#-immobile-code
-(define-full-call static-tail-call-named :direct :tail nil)
 
 (define-full-call call-variable nil :fixed t)
 (define-full-call multiple-call-variable nil :unknown t)
@@ -877,32 +893,20 @@
 
 (define-full-call unboxed-call-named t :unboxed nil)
 (define-full-call fixed-unboxed-call-named t :unboxed nil :fixed)
+(define-full-call fixed-multiple-call-named t :unknown nil :fixed)
 
 ;;; Call NAME "directly" meaning in a single JMP or CALL instruction,
 ;;; if possible (without loading RAX)
 (defun emit-direct-call (name instruction node step-instrumenting)
-      ;; a :STATIC-CALL fixup is the address of the entry point of
-      ;; the function itself, and a :FDEFN-CALL fixup is the address
-      ;; of the JMP instruction embedded in the header for the named FDEFN.
-  (when (static-fdefn-offset name)
-    (let ((fixup (make-fixup name :static-call)))
-      (return-from emit-direct-call
-        (inst* instruction (if (sb-c::code-immobile-p node) fixup (ea fixup))))))
-  (let* ((fixup (make-fixup name :fdefn-call))
-         (target
-              (if (and (sb-c::code-immobile-p node)
-                       (not step-instrumenting))
-                  fixup
-                  (progn
-                    ;; RAX-TN was not declared as a temp var,
-                    ;; however it's sole purpose at this point is
-                    ;; for function call, so even if it was used
-                    ;; to compute a stack argument, it's free now.
-                    ;; If the call hits the undefined fun trap,
-                    ;; RAX will get loaded regardless.
-                    (inst mov rax-tn fixup)
-                    rax-tn))))
-    (inst* instruction target)))
+  (cond (step-instrumenting
+         ;; If step-instrumenting, then RAX points to the linkage table cell
+         (inst* instruction (ea rax-tn)))
+        ((sb-c::code-immobile-p node)
+         (inst* instruction (rip-relative-ea (linkage-cell-fixup name node))))
+        (t
+         ;; get the linkage table base into RAX
+         (inst mov rax-tn (thread-slot-ea sb-vm::thread-linkage-table-slot))
+         (inst* instruction (ea (linkage-cell-fixup name node) rax-tn)))))
 
 ;;; Invoke the function-designator FUN.
 (defun tail-call-unnamed (fun type vop)
@@ -1327,11 +1331,17 @@
   (:results (result :scs (descriptor-reg)))
   (:node-var node)
   (:generator 20
+#|
+    ;; TODO: if instrumenting, just revert to the older way of precomputing
+    ;; a size rather than scaling by 8 in ALLOCATION so that we don't have
+    ;; to scale and unscale.
     ;; Compute the number of bytes to allocate
     (let ((shift (- (1+ word-shift) n-fixnum-tag-bits)))
       (if (location= count rcx)
           (inst shl :dword rcx shift)
           (inst lea :dword rcx (ea nil count (ash 1 shift)))))
+|#
+    (move rcx count :dword)
     ;; Setup for the CDR of the last cons (or the entire result) being NIL.
     (inst mov result nil-value)
     (cond ((not (member :allocation-size-histogram sb-xc:*features*))
@@ -1339,13 +1349,22 @@
           (t ; jumps too far for JRCXZ sometimes
            (inst test rcx rcx)
            (inst jmp :z done)))
-    (unless (node-stack-allocate-p node)
-      (instrument-alloc +cons-primtype+ rcx node (list value dst) thread-tn))
+    (when (and (not (node-stack-allocate-p node)) (instrument-alloc-policy-p node))
+      (inst shl :dword rcx word-shift) ; compute byte count
+      (instrument-alloc +cons-primtype+ rcx node (list value dst) thread-tn)
+      (inst shr :dword rcx word-shift)) ; undo the computation
     (pseudo-atomic (:elide-if (node-stack-allocate-p node) :thread-tn thread-tn)
        ;; Produce an untagged pointer into DST
-       (if (node-stack-allocate-p node)
-           (stack-allocation rcx 0 dst)
-           (allocation +cons-primtype+ rcx 0 dst node value thread-tn
+      (let ((scale
+             (cond ((node-stack-allocate-p node)
+                    ;; LEA on RSP would be ok but we'd need to negate RCX first, then un-negate
+                    ;; to compute the final cons, then negate again. So use SHL and SUB instead.
+                    (inst shl :dword rcx word-shift)
+                    (stack-allocation rcx 0 dst)
+                    1)
+                   (t
+                    (allocation +cons-primtype+ rcx 0 dst node value thread-tn
+                       :scale 8
                        :overflow
                        (lambda ()
                          (inst push rcx)
@@ -1354,13 +1373,16 @@
                           'call (if (system-tlab-p 0 node) 'sys-listify-&rest 'listify-&rest)
                           node)
                          (inst pop result)
-                         (inst jmp leave-pa))))
+                         (inst jmp alloc-done)))
+                    8))))
        ;; Recalculate DST as a tagged pointer to the last cons
-       (inst lea dst (ea (- list-pointer-lowtag (* cons-size n-word-bytes)) dst rcx))
-       (inst shr :dword rcx (1+ word-shift)) ; convert bytes to number of cells
+       (inst lea dst (ea (- list-pointer-lowtag (* cons-size n-word-bytes)) dst rcx scale))
+       ;; scale=8 implies RCX counts ncells (as a fixnum) therefore just untag it.
+       ;; scale=1 implies RCX counts nbytes therefore ncells = RCX/16
+       (inst shr :dword rcx (if (= scale 8) n-fixnum-tag-bits (1+ word-shift))))
        ;; The rightmost arguments are at lower addresses.
        ;; Start by indexing the last argument
-       (inst neg rcx) ; :QWORD because it's a signed number
+       (inst neg rcx) ; :QWORD because it's negative
        LOOP
        ;; Grab one value and store into this cons. Use RCX as an index into the
        ;; vector of values in CONTEXT, but add 8 because CONTEXT points exactly at
@@ -1374,9 +1396,9 @@
        (storew value dst cons-car-slot list-pointer-lowtag)
        (inst mov result dst) ; preserve the value to put in the CDR of the preceding cons
        (inst sub dst (* cons-size n-word-bytes)) ; get the preceding cons
-       (inst inc rcx) ; :QWORD because it's a signed number
+       (inst inc rcx) ; :QWORD because it's negative
        (inst jmp :nz loop)
-       LEAVE-PA)
+       ALLOC-DONE)
     DONE))
 
 ;;; Return the location and size of the &MORE arg glob created by
@@ -1416,11 +1438,16 @@
   (:temporary (:sc unsigned-reg :offset rbx-offset) temp)
   (:info min max)
   (:vop-var vop)
+  (:node-var node)
   (:save-p :compute-only)
   (:generator 3
+    RESTART
     ;; NOTE: copy-more-arg expects this to issue a CMP for min > 1
-    (let ((err-lab
-            (generate-error-code vop 'invalid-arg-count-error nargs)))
+    (let* ((*location-context* (and max
+                                    (policy node (> debug 1))
+                                    (cons (make-restart-location RESTART) max)))
+           (err-lab
+             (generate-error-code vop 'invalid-arg-count-error nargs)))
       (cond ((not min)
              (if (zerop max)
                  (inst test :dword nargs nargs)
