@@ -18,7 +18,7 @@
 #include <string.h>
 #include <sys/file.h>
 
-#include "sbcl.h"
+#include "genesis/sbcl.h"
 #ifdef LISP_FEATURE_WIN32
 #include "pthreads_win32.h"
 #else
@@ -28,8 +28,6 @@
 #include "os.h"
 #include "core.h"
 #include "globals.h"
-#include "save.h"
-#include "dynbind.h"
 #include "lispregs.h"
 #include "validate.h"
 #include "gc.h"
@@ -44,6 +42,7 @@
 #ifdef LISP_FEATURE_SB_CORE_COMPRESSION
 # include <zstd.h>
 #endif
+#define COMPRESSION_LEVEL_NONE INT_MIN
 
 #define GENERAL_WRITE_FAILURE_MSG "error writing to core file"
 
@@ -139,6 +138,7 @@ write_bytes_to_file(FILE * file, char *addr, size_t bytes, int compression)
                bytes, total_written, compression);
 
         ZSTD_freeCStream(stream);
+        free(buf);
 #endif
     } else {
 #ifdef LISP_FEATURE_SB_CORE_COMPRESSION
@@ -155,7 +155,7 @@ write_bytes_to_file(FILE * file, char *addr, size_t bytes, int compression)
 };
 
 static long write_bytes(FILE *file, char *addr, size_t bytes,
-                        os_vm_offset_t file_offset, int compression, int align)
+                        os_vm_offset_t file_offset, int compression)
 {
     ftell_type here, data;
 
@@ -169,7 +169,7 @@ static long write_bytes(FILE *file, char *addr, size_t bytes,
     here = FTELL(file);
     FSEEK(file, 0, SEEK_END);
     data = ALIGN_UP(FTELL(file), os_vm_page_size);
-    if (align) FSEEK(file, data, SEEK_SET);
+    FSEEK(file, data, SEEK_SET);
     write_bytes_to_file(file, addr, bytes, compression);
     FSEEK(file, here, SEEK_SET);
     return ((data - file_offset) / os_vm_page_size) - 1;
@@ -182,7 +182,7 @@ output_space(FILE *file, int id, lispobj *addr, lispobj *end,
 {
     size_t words, bytes, data, compressed_flag;
     static char *names[] = {NULL, "dynamic", "static", "read-only",
-                            "fixedobj", "text"};
+      "fixedobj", "text", "permgen"};
 
     compressed_flag
             = ((core_compression_level != COMPRESSION_LEVEL_NONE)
@@ -212,7 +212,7 @@ output_space(FILE *file, int id, lispobj *addr, lispobj *end,
      * That seems quite bogus to operate on bytes that the caller didn't promise were OK
      * to be saved out (and didn't contain, say, a password and social security number) */
     data = write_bytes(file, (char *)addr, ALIGN_UP(bytes, os_vm_page_size),
-                       file_offset, core_compression_level, 1);
+                       file_offset, core_compression_level);
 
     write_lispobj(data, file);
     write_lispobj((uword_t)addr, file);
@@ -229,10 +229,9 @@ open_core_for_saving(char *filename)
     return fopen(filename, "wb");
 }
 
-void unwind_binding_stack()
+static void unwind_binding_stack(struct thread* th)
 {
     bool verbose = !lisp_startup_options.noinform;
-    struct thread *th = all_threads;
 
     /* Smash the enclosing state. (Once we do this, there's no good
      * way to go back, which is a sufficient reason that this ends up
@@ -286,10 +285,21 @@ bool save_to_filehandle(FILE *file, char *filename, lispobj init_function,
     int string_words = ALIGN_UP(stringlen, sizeof (core_entry_elt_t))
         / sizeof (core_entry_elt_t);
     int pad = string_words * sizeof (core_entry_elt_t) - stringlen;
-    /* Write 3 word entry header: a word for entry-type-code, a word for
-     * the total length in words, and a word for the string length */
+    /* Write 5 word entry header: a word for entry-type-code, the length in words,
+     * the GC enum, the address of NIL, and the string length */
     write_lispobj(BUILD_ID_CORE_ENTRY_TYPE_CODE, file);
-    write_lispobj(3 + string_words, file);
+    write_lispobj(5 + string_words, file);
+#ifdef LISP_FEATURE_GENCGC
+    write_lispobj(1, file);
+#endif
+#ifdef LISP_FEATURE_MARK_REGION_GC
+    write_lispobj(2, file);
+#endif
+#ifdef LISP_FEATURE_RELOCATABLE_STATIC_SPACE
+    write_lispobj(0, file);
+#else
+    write_lispobj(NIL, file);
+#endif
     write_lispobj(stringlen, file);
     int nwrote = fwrite(build_id, 1, stringlen, file);
     /* Write padding bytes to align to core_entry_elt_t */
@@ -297,30 +307,71 @@ bool save_to_filehandle(FILE *file, char *filename, lispobj init_function,
     if (nwrote != (int)(sizeof (core_entry_elt_t) * string_words))
         perror(GENERAL_WRITE_FAILURE_MSG);
 
+#ifdef LISP_FEATURE_LINKAGE_SPACE
+    // Lisp linkage space precedes the general space directory
+    int i;
+    extern void illegal_linkage_space_call();
+    /* The C runtime is theoretically position-independent so don't write out the address
+     * of the unused entry sentinel. The affected elements needn't be restored on restart.
+     * (Maybe add a renumbering pass in Lisp to ensure the table is 100% dense) */
+    for (i=0; i<linkage_table_count; ++i)
+        if (linkage_space[i] == (uword_t)illegal_linkage_space_call)
+            linkage_space[i] = 0;
+    int nbytes = ALIGN_UP(linkage_table_count<<WORD_SHIFT, BACKEND_PAGE_BYTES);
+    if (!lisp_startup_options.noinform)
+        printf("writing %lu bytes from the %s space at %p\n",
+               (long unsigned)nbytes, "linkage", linkage_space);
+
+    write_lispobj(LISP_LINKAGE_SPACE_CORE_ENTRY_TYPE_CODE, file);
+    write_lispobj(5, file); // number of words in this core header entry
+    write_lispobj(linkage_table_count, file);
+    sword_t data_page =
+        write_bytes(file, (char*)linkage_space,
+                    nbytes, core_start_pos, COMPRESSION_LEVEL_NONE);
+    write_lispobj(data_page, file);
+    write_lispobj(0, file); // address of ELF-based linkage entries
+#endif
+
     write_lispobj(DIRECTORY_CORE_ENTRY_TYPE_CODE, file);
-    write_lispobj(/* (word count = N spaces described by 5 words each, plus the
-          * entry type code, plus this count itself) */
-         (5 * MAX_CORE_SPACE_ID) + 2, file);
+    ftell_type spacecount_pos = FTELL(file);
+    write_lispobj(0, file); // placeholder
+
+    int count = 0;
     output_space(file,
                  STATIC_CORE_SPACE_ID,
                  (lispobj *)STATIC_SPACE_START,
                  static_space_free_pointer,
                  core_start_pos,
-                 core_compression_level);
+                 core_compression_level), ++count;
+#ifdef LISP_FEATURE_PERMGEN
+    {
+#define REMEMBERED_BIT (uword_t)0x80000000
+        lispobj* where = (void*)PERMGEN_SPACE_START;
+        // clear every object's bit
+        for ( ; where < permgen_space_free_pointer ; where += object_size(where) )
+            *where &= ~REMEMBERED_BIT;
+    }
+    output_space(file,
+                 PERMGEN_CORE_SPACE_ID,
+                 (lispobj *)PERMGEN_SPACE_START,
+                 permgen_space_free_pointer,
+                 core_start_pos,
+                 core_compression_level), ++count;
+#endif
 #ifdef LISP_FEATURE_DARWIN_JIT
     output_space(file,
                  STATIC_CODE_CORE_SPACE_ID,
                  (lispobj *)STATIC_CODE_SPACE_START,
                  static_code_space_free_pointer,
                  core_start_pos,
-                 core_compression_level);
+                 core_compression_level), ++count;
 #endif
     output_space(file,
                  DYNAMIC_CORE_SPACE_ID,
                  (void*)DYNAMIC_SPACE_START,
                  (void*)dynamic_space_highwatermark(),
                  core_start_pos,
-                 core_compression_level);
+                 core_compression_level), ++count;
     /* FreeBSD really doesn't like to give you the address you asked for
      * on dynamic space. We have a better chance at satisfying the request
      * if we haven't already mapped R/O space below it. */
@@ -329,14 +380,15 @@ bool save_to_filehandle(FILE *file, char *filename, lispobj init_function,
                  (lispobj *)READ_ONLY_SPACE_START,
                  read_only_space_free_pointer,
                  core_start_pos,
-                 core_compression_level);
+                 core_compression_level), ++count;
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
     output_space(file,
                  IMMOBILE_FIXEDOBJ_CORE_SPACE_ID,
                  (lispobj *)FIXEDOBJ_SPACE_START,
                  fixedobj_free_pointer,
                  core_start_pos,
-                 core_compression_level);
+                 core_compression_level), ++count;
+#endif
     // Leave this space for last! Things are easier when splitting a core into
     // code and non-code if we don't have to compensate for removal of pages.
     // i.e. if code resided between dynamic and fixedobj space, then dynamic
@@ -346,8 +398,7 @@ bool save_to_filehandle(FILE *file, char *filename, lispobj init_function,
                  (lispobj *)TEXT_SPACE_START,
                  text_space_highwatermark,
                  core_start_pos,
-                 core_compression_level);
-#endif
+                 core_compression_level), ++count;
 
     write_lispobj(INITIAL_FUN_CORE_ENTRY_TYPE_CODE, file);
     write_lispobj(3, file);
@@ -356,35 +407,40 @@ bool save_to_filehandle(FILE *file, char *filename, lispobj init_function,
 #ifdef LISP_FEATURE_GENERATIONAL
     {
         extern void gc_store_corefile_ptes(struct corefile_pte*);
-        size_t true_size = next_free_page * sizeof(struct corefile_pte);
-        size_t aligned_size = ALIGN_UP(true_size, N_WORD_BYTES);
+        sword_t bitmapsize = 0;
+#ifdef LISP_FEATURE_MARK_REGION_GC
+        bitmapsize = bitmap_size(next_free_page);
+#endif
+        size_t ptes_nbytes = next_free_page * sizeof(struct corefile_pte);
+        size_t aligned_size = ALIGN_UP((bitmapsize+ptes_nbytes), N_WORD_BYTES);
         char* data = successful_malloc(aligned_size);
         // Zeroize the final few bytes of data that get written out
         // but might be untouched by gc_store_corefile_ptes().
         memset(data + aligned_size - N_WORD_BYTES, 0, N_WORD_BYTES);
-        gc_store_corefile_ptes((struct corefile_pte*)data);
+#ifdef LISP_FEATURE_MARK_REGION_GC
+        memcpy(data, allocation_bitmap, bitmapsize);
+#endif
+        gc_store_corefile_ptes((struct corefile_pte*)(data + bitmapsize));
         write_lispobj(PAGE_TABLE_CORE_ENTRY_TYPE_CODE, file);
         write_lispobj(6, file); // number of words in this core header entry
         write_lispobj(gc_card_table_nbits, file);
         write_lispobj(next_free_page, file);
         write_lispobj(aligned_size, file);
         sword_t offset = write_bytes(file, data, aligned_size, core_start_pos,
-                                     COMPRESSION_LEVEL_NONE, 1);
-#ifdef LISP_FEATURE_MARK_REGION_GC
-        sword_t size = bitmap_size(next_free_page);
-        extern uword_t *allocation_bitmap;
-        write_bytes(file, (char*)allocation_bitmap, size, core_start_pos, COMPRESSION_LEVEL_NONE, 0);
-#endif
+                                     COMPRESSION_LEVEL_NONE);
         write_lispobj(offset, file);
     }
 #endif
 
     write_lispobj(END_CORE_ENTRY_TYPE_CODE, file);
+    FSEEK(file, spacecount_pos, SEEK_SET);
+    // 5 = length of (struct ndir_entry) in words, plus 2 fixed words
+    write_lispobj(2 + count * 5, file);
 
     /* Write a trailing header, ignored when parsing the core normally.
      * This is used to locate the start of the core when the runtime is
      * prepended to it. */
-    fseek(file, 0, SEEK_END);
+    FSEEK(file, 0, SEEK_END);
 
     if (1 != fwrite(&core_start_pos, sizeof(os_vm_offset_t), 1, file)) {
         perror("Error writing core starting position to file");
@@ -555,34 +611,6 @@ prepare_to_save(char *filename, bool prepend_runtime, void **runtime_bytes,
     return file;
 }
 
-#ifdef LISP_FEATURE_CHENEYGC
-bool save(char *filename, lispobj init_function, bool prepend_runtime,
-          bool save_runtime_options, bool compressed, int compression_level,
-          int application_type)
-{
-    FILE *file;
-    void *runtime_bytes = NULL;
-    size_t runtime_size;
-
-    file = prepare_to_save(filename, prepend_runtime, &runtime_bytes, &runtime_size);
-    if (file == NULL)
-        return 1;
-
-    if (prepend_runtime)
-        save_runtime_to_filehandle(file, runtime_bytes, runtime_size, application_type);
-
-    /* This unwinding is necessary for proper restoration of the
-     * symbol-value slots to their toplevel values, but it occurs
-     * too late to remove old references from the binding stack.
-     * There's probably no safe way to do that from Lisp */
-    unwind_binding_stack();
-    os_unlink_runtime();
-    return save_to_filehandle(file, filename, init_function, prepend_runtime,
-                              save_runtime_options,
-                              compressed ? compressed : COMPRESSION_LEVEL_NONE);
-}
-#endif
-
 #include "incremental-compact.h"
 
 /* Things to do before doing a final GC before saving a core.
@@ -594,12 +622,11 @@ bool save(char *filename, lispobj init_function, bool prepend_runtime,
  * + Instances on unboxed pages need to have their layout pointer visited,
  *   so all pages have to be turned to boxed.
  */
-static void prepare_dynamic_space_for_final_gc()
+static void prepare_dynamic_space_for_final_gc(struct thread* thread)
 {
     page_index_t i;
 
     prepare_immobile_space_for_final_gc();
-    /* TODO: external full compactor thingy for MR */
     for (i = 0; i < next_free_page; i++) {
 #ifndef LISP_FEATURE_MARK_REGION_GC
         // Compaction requires that we permit large objects to be copied henceforth.
@@ -620,6 +647,10 @@ static void prepare_dynamic_space_for_final_gc()
 #endif
         }
     }
+#ifdef LISP_FEATURE_PERMGEN
+    extern void remember_all_permgen();
+    remember_all_permgen();
+#endif
 #ifdef LISP_FEATURE_MARK_REGION_GC
     for (generation_index_t g = 1; g <= PSEUDO_STATIC_GENERATION; g++) {
       generations[0].bytes_allocated += generations[g].bytes_allocated;
@@ -639,7 +670,6 @@ static void prepare_dynamic_space_for_final_gc()
 #ifdef LISP_FEATURE_SB_THREAD
     // Avoid tenuring of otherwise-dead objects referenced by
     // dynamic bindings which disappear on image restart.
-    struct thread *thread = get_sb_vm_thread();
     char *start = (char*)&thread->lisp_thread;
     char *end = (char*)thread + dynamic_values_bytes;
     memset(start, 0, end-start);
@@ -742,17 +772,7 @@ gc_and_save(char *filename, bool prepend_runtime, bool purify,
     struct thread *thread = get_sb_vm_thread();
     gc_close_thread_regions(thread, 0);
     gc_close_collector_regions(0);
-#ifdef LISP_FEATURE_MARK_REGION_GC
-    /* Do a minor GC to instate allocation bitmap for new objects.
-     * This is needed to make heap walking in move_rospace_to_dynamic
-     * work. */
-    collect_garbage(0);
-#endif
-    move_rospace_to_dynamic(0);
-    pre_verify_gen_0 = 1;
-    prepare_immobile_space_for_final_gc(); // once is enough
-    prepare_dynamic_space_for_final_gc();
-    unwind_binding_stack();
+    unwind_binding_stack(thread);
     // Avoid tenuring of otherwise-dead objects referenced by bindings which
     // disappear on image restart. This must occcur *after* unwind_binding_stack()
     // because unwinding moves values from the binding stack into TLS.
@@ -762,6 +782,17 @@ gc_and_save(char *filename, bool prepend_runtime, bool purify,
     // After zeroing, make sure PINNED_OBJECTS is a list again.
     write_TLS(PINNED_OBJECTS, NIL, thread);
 
+    pre_verify_gen_0 = 1;
+#ifdef LISP_FEATURE_MARK_REGION_GC
+    /* Do a minor GC to instate allocation bitmap for new objects.
+     * This is needed to make heap walking in move_rospace_to_dynamic
+     * work. */
+    collect_garbage(0);
+#endif
+    move_rospace_to_dynamic(0);
+    prepare_immobile_space_for_final_gc(); // once is enough
+    prepare_dynamic_space_for_final_gc(thread);
+
     save_lisp_gc_iteration = 1;
 #ifndef LISP_FEATURE_MARK_REGION_GC
     gencgc_alloc_start_page = next_free_page;
@@ -769,7 +800,7 @@ gc_and_save(char *filename, bool prepend_runtime, bool purify,
     collect_garbage(0);
     verify_heap(0, VERIFY_POST_GC);
 
-    THREAD_JIT(0);
+    THREAD_JIT_WP(0);
 
     // We always coalesce copyable numbers. Additional coalescing is done
     // only on request, in which case a message is shown (unless verbose=0).
@@ -787,7 +818,7 @@ gc_and_save(char *filename, bool prepend_runtime, bool purify,
     collect_garbage(1+PSEUDO_STATIC_GENERATION);
     prepare_readonly_space(purify, 0);
     if (verbose) { printf("[performing final GC..."); fflush(stdout); }
-    prepare_dynamic_space_for_final_gc();
+    prepare_dynamic_space_for_final_gc(thread);
     save_lisp_gc_iteration = 2;
     gencgc_alloc_start_page = 0;
     collect_garbage(0);
@@ -798,7 +829,7 @@ gc_and_save(char *filename, bool prepend_runtime, bool purify,
     if (verbose)
         printf(" done]\n");
 
-    THREAD_JIT(0);
+    THREAD_JIT_WP(0);
     // Scrub remaining garbage
     extern void zero_all_free_ranges(void);
     zero_all_free_ranges();
