@@ -584,3 +584,42 @@
               ((unsigned-byte 8) a))
      (floor a b))
    (values (integer -255 0) (integer -9 0) &optional)))
+
+(with-test (:name :logbitp-on-integers)
+  (assert (not (ctu:ir1-named-calls `(lambda (x)
+                                       (logbitp 20 x))))))
+(with-test (:name :bt-negative-bit)
+  (checked-compile-and-assert
+   ()
+   `(lambda (c)
+     (declare ((signed-byte 64) c))
+     (logtest c -2199023255553))
+   ((-2049990302793354782) t)
+   ((0) nil)
+   (((ash 1 41)) nil))
+  (checked-compile-and-assert
+   ()
+   `(lambda (b)
+      (declare (fixnum b))
+      (logior b -4611686018427387905))
+   ((-6) -1)))
+
+(with-test (:name :float-cmp)
+  (checked-compile-and-assert
+      ()
+      `(lambda (a b)
+         (declare ((unsigned-byte 20) a)
+                  (float b))
+         (< a b))
+    ((6 4.0) nil)
+    ((1 1.1) t)))
+
+(with-test (:name :complex+non-complex-type)
+  (assert-type
+   (lambda (a)
+     (+ a #c(1.0 3.0)))
+   (or (complex single-float) (complex double-float)))
+  (assert-type
+   (lambda (a)
+     (* a #c(1d0 0d0)))
+    (complex double-float)))
