@@ -943,7 +943,7 @@
    (lambda (x y)
      (when (integerp (+ x y))
        x))
-   (or number null))
+   (or rational (complex rational) null))
   (assert-type
    (lambda (x y)
      (declare (real y))
@@ -966,7 +966,12 @@
               (fixnum x))
      (when (typep (+ m x) '(integer 2 4))
        x))
-   (or (integer -2 2) null)))
+   (or (integer -2 2) null))
+  (assert-type
+   (lambda (m)
+     (the (integer 0 30) (+ m m))
+     m)
+   rational))
 
 (with-test (:name :-back)
   (assert-type
@@ -983,7 +988,22 @@
    (lambda (x)
      (when (> (- 3 x) 10)
        x))
-   (or real null)))
+   (or real null))
+  (assert-type
+   (lambda (x)
+     (when (floatp (- x))
+       x))
+   (or float null))
+  (assert-type
+   (lambda (x)
+     (when (typep (- x) '(integer 1 2))
+       x))
+   (or (integer -2 -1) null))
+  (assert-type
+   (lambda (m)
+     (the real (+ m m))
+     m)
+   real))
 
 (with-test (:name :*back)
   (assert-type
@@ -996,7 +1016,7 @@
      (declare ((real 0 3) m))
      (when (typep (* x m) '(integer 0 10))
        x))
-   (or number null))
+   (or (or rational (complex rational) null) null))
   (assert-type
    (lambda (x)
      (when (> (* x 3) 10)
@@ -1008,7 +1028,17 @@
               (integer m))
      (when (typep (* x m) 'integer)
        x))
-   (or rational null)))
+   (or rational null))
+  (assert-type
+   (lambda (m)
+     (the (integer 0 30) (* m m))
+     m)
+   integer)
+  (assert-type
+   (lambda (m)
+     (when (typep (+ m 1) 'float)
+       m))
+   (or null float)))
 
 (with-test (:name :ignore-hairy-types)
   (checked-compile
@@ -1708,3 +1738,142 @@
        (let ((j (remove-if #'evenp x)))
          (length j))))
    (or null (mod 5))))
+
+(with-test (:name :zero-length-check-bound)
+  (assert (= (count 'sb-kernel:%check-bound
+                    (ctu:ir1-named-calls
+                     `(lambda (x)
+                        (declare (simple-string x))
+                        (unless (string= x "")
+                          (aref x 0)))
+                     nil))
+             0)))
+
+(with-test (:name :concatenate-length)
+  (assert-type
+   (lambda (x y)
+     (declare ((simple-vector 10) x)
+              (optimize (debug 2)))
+     (let ((x (concatenate 'vector x y)))
+       (length x)))
+   (integer 10 (#.array-dimension-limit)))
+  (assert-type
+   (lambda (x)
+     (declare (optimize (debug 2)))
+     (let ((x (concatenate 'vector '(1 2 3) (subseq x 1))))
+       (length x)))
+   (integer 3 (#.array-dimension-limit)))
+  (assert-type
+   (lambda (x)
+     (declare ((simple-vector 3) x)
+              (optimize (debug 2)))
+     (let ((x (concatenate 'vector x (if * "abc" "abcd"))))
+       (length x)))
+   (integer 6 7))
+  (assert-type
+   (lambda (x)
+     (declare (optimize (debug 2)))
+     (let ((x (concatenate 'vector "00" (subseq x 0 2))))
+       (length x)))
+   (integer 4 4)))
+
+(with-test (:name :inherit-length-var)
+  (assert (not (ctu:ir1-named-calls
+                `(lambda (x y n)
+                   (declare (optimize (space 0))
+                            (fixnum n)
+                            (simple-base-string y))
+                   (when (< n (length y))
+                     (find x y :start n))))))
+  (assert-type
+   (lambda (x y)
+     (declare (simple-vector x))
+     (if (< y (length x))
+         (let ((m (length x)))
+           (if (< y m)
+               t
+               m))
+         t))
+   (eql t)))
+
+(with-test (:name :multiple-back)
+  (assert-type
+   (lambda (m)
+     (the (integer 0 30) (+ (* m 2) 2))
+     m)
+   (rational -1 14)))
+
+(with-test (:name :truncate-back)
+  (assert-type
+   (lambda (m d)
+     (declare (integer m)
+              ((integer 3 5) d))
+     (if (typep (truncate m d) '(integer -5 10))
+         m))
+   (or (integer -29 54) null))
+  (assert-type
+   (lambda (m d)
+     (declare (integer m)
+              ((integer 3 5) d))
+     (if (typep (truncate m d) '(integer 0 10))
+         m))
+   (or (integer -4 54) null))
+  (assert-type
+   (lambda (m d)
+     (declare (integer m)
+              ((integer 3 5) d))
+     (if (typep (truncate m d) '(integer 1 10))
+         m))
+   (or (integer 3 54) null))
+  (assert-type
+   (lambda (m d)
+     (declare (integer m)
+              ((integer 3 5) d))
+     (if (typep (truncate m d) '(integer -5 0))
+         m))
+   (or (integer -29 4) null))
+  (assert-type
+   (lambda (m d)
+     (declare (integer m)
+              ((integer 3 5) d))
+     (if (typep (truncate m d) '(integer -5 -2))
+         m))
+   (or (integer -29 -6) null))
+  (assert-type
+   (lambda (m d)
+     (declare (integer m)
+              ((integer -4 3) d))
+     (if (typep (truncate m d) '(integer -5 2))
+         m))
+   (or (integer -17 23) null))
+
+  (assert-type
+   (lambda (m)
+     (if (typep (nth-value 1 (truncate m)) 'double-float)
+         m))
+   (or double-float null))
+  (assert-type
+   (lambda (m)
+     (if (typep (truncate m) 'fixnum)
+         m))
+   (or real null))
+  (assert-type
+   (lambda (m)
+     (if (typep (nth-value 1 (truncate m 2.0)) 'double-float)
+         m))
+   (or double-float null)))
+
+(with-test (:name :ignore-delays)
+  (assert-type
+   (lambda (x)
+     (declare (optimize debug))
+     (when (typep (nth-value 1 (truncate x 1)) 'float)
+       x))
+   (or null real)))
+
+(with-test (:name :not-eq-eql)
+  (assert-type
+   (lambda (a b)
+     (and (eql a b)
+          (not (eq a b))))
+   boolean))
