@@ -255,10 +255,12 @@
                                       collect (sb-thread:make-thread #'start-run)))
             #-sb-thread (loop repeat 10 collect (start-run))))))
 
-(with-test (:name (run-program :pty-stream) :fails-on :win32)
+(with-test (:name (run-program :pty-stream)
+            :fails-on :win32
+            :broken-on :darwin)
   (let (process
         stream)
-    (assert (equal "OK"
+    (assert (search "OK"
                    (handler-bind
                        ((timeout (lambda (c)
                                    c
@@ -266,17 +268,14 @@
                                            (when stream
                                              (get-output-stream-string stream))))))
                      (with-timeout 60
-                       (subseq
-                        (with-output-to-string (s)
-                          (setf stream s)
-                          (setf process
-                                (run-program "/bin/sh" '("-c" "echo OK; exit 42") :pty s
-                                                                                  :wait nil))
-                          (process-wait process)
-                          (assert (= (process-exit-code process) 42))
-                          s)
-                        0
-                        2)))))))
+                       (with-output-to-string (s)
+                         (setf stream s)
+                         (setf process
+                               (run-program "/bin/sh" '("-c" "echo OK; exit 42") :pty s
+                                                                                 :wait nil))
+                         (process-wait process)
+                         (assert (= (process-exit-code process) 42))
+                         s)))))))
 
 ;; Check whether RUN-PROGRAM puts its child process into the foreground
 ;; when stdin is inherited. If it fails to do so we will receive a SIGTTIN.
@@ -435,7 +434,7 @@
 
 (with-test (:name (run-program :malloc-deadlock)
             :broken-on :sb-safepoint
-            :skipped-on (or :ubsan (not :sb-thread) :win32))
+            :skipped-on (or :ubsan (not :sb-thread) :win32 :android))
   (let* (stop
          (delay-between-gc
           (or #+freebsd
